"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_leveldown_1 = require("abstract-leveldown");
const heapq_1 = __importDefault(require("heapq"));
const ltgt_1 = __importDefault(require("ltgt"));
const callback_promise_1 = require("./util/callback-promise");
const concat_1 = require("./util/concat");
const leveldown_opts_1 = require("./util/leveldown-opts");
class LevelDOWNMountIterator extends abstract_leveldown_1.AbstractIterator {
    constructor(db, options) {
        super(options);
        this.db = db;
        this._options = leveldown_opts_1.normalize_options(options);
        this._test = leveldown_opts_1.test_from_options(this._options);
        // Sorted mount locations
        const _test_prefix = leveldown_opts_1.test_prefix_from_options(this._options);
        const _mounts = Object.keys(db._mounts).filter(_test_prefix);
        // Filter by mounts which could overlap with the iteration
        //  (the root mount and any mount which satisfies the range filter)
        this._mounts = _mounts.map((mount) => {
            let opts = Object.assign({}, this._options);
            if (mount !== '') {
                if (opts.gt !== undefined && opts.gt.slice(0, mount.length) === mount) {
                    opts.gt = opts.gt.slice(mount.length);
                    opts.lt = opts.lte = undefined;
                }
                else if (opts.gte !== undefined && opts.gte.slice(0, mount.length) === mount) {
                    opts.gte = opts.gte.slice(mount.length);
                    opts.lt = opts.lte = undefined;
                }
                else if (opts.lt !== undefined && opts.lt.slice(0, mount.length) === mount) {
                    opts.lt = opts.lt.slice(mount.length);
                    opts.gt = opts.gte = undefined;
                }
                else if (opts.lte !== undefined && opts.lte.slice(0, mount.length) === mount) {
                    opts.lte = opts.lte.slice(mount.length);
                    opts.gt = opts.gte = undefined;
                }
                else {
                    opts.gt = opts.gte = opts.lt = opts.lte = undefined;
                }
            }
            return ({
                mount: mount,
                iterator: db._mounts[mount].iterator(opts),
            });
        });
        this._iterator = this.iterator();
    }
    iterator() {
        return __asyncGenerator(this, arguments, function* iterator_1() {
            let Q = [];
            let done = 0;
            // Compare by next iterable key
            const _cmp = this._options.reverse ? ((a, b) => ltgt_1.default.compare(a.next.key, b.next.key) > 0) : ((a, b) => ltgt_1.default.compare(a.next.key, b.next.key) < 0);
            // Get next iterable for given mount
            const _next = (mount) => __awaiter(this, void 0, void 0, function* () {
                const { key, value } = yield callback_promise_1.ErrorKeyValueCallbackToPromise(mount.iterator.next.bind(mount.iterator));
                const next_mount = Object.assign({}, mount, { done: key === undefined && value === undefined, next: {
                        key: key === undefined ? undefined : concat_1.concat(mount.mount, key),
                        value: value
                    } });
                return next_mount;
            });
            // Initialize heapq with first value from all mounts
            for (const mount of this._mounts) {
                const next_mount = yield __await(_next(mount));
                if (!next_mount.done)
                    heapq_1.default.push(Q, next_mount, _cmp);
            }
            // Work all iterators
            while (Q.length > 0 && done++ < this._options.limit) {
                const mount = heapq_1.default.pop(Q, _cmp);
                if (this._test(mount.next.key))
                    yield yield __await(mount.next);
                const next_mount = yield __await(_next(mount));
                if (!next_mount.done)
                    heapq_1.default.push(Q, next_mount, _cmp);
            }
        });
    }
    // Get the next key/value reported by all relevant mounts
    _next(callback) {
        this._iterator.next().then(({ done, value }) => {
            if (done)
                process.nextTick(callback);
            else
                process.nextTick(callback, null, value.key, value.value);
        }).catch((err) => process.nextTick(callback, err));
    }
    // Cleanly end this iterator by ending any remaining mount iterators
    _end(callback) {
        if (this._mounts.length === 0) {
            // No mounts left to end, we're done
            return process.nextTick(callback);
        }
        else {
            // Use the first mount on the queue
            const mount = this._mounts[0];
            // End the mounts iterator
            return process.nextTick(mount.iterator.end.bind(mount.iterator), (err) => {
                // Shift the mount off the queue
                this._mounts.shift();
                // Continue ending this iterator reporting any error
                process.nextTick(this._end.bind(this), (_err) => process.nextTick(callback, _err || err));
            });
        }
    }
}
exports.LevelDOWNMountIterator = LevelDOWNMountIterator;
//# sourceMappingURL=level-mount-iterator.js.map