"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = __importStar(require("assert"));
const levelup_1 = __importDefault(require("levelup"));
const reverse_promise_1 = require("../util/reverse-promise");
const stream_promise_1 = require("../util/stream-promise");
function CustomLevelDOWNTestSuite(desc, LevelDOWNFactory) {
    function withEmptyStore() {
        return __awaiter(this, void 0, void 0, function* () {
            return levelup_1.default(LevelDOWNFactory());
        });
    }
    function withStore() {
        return __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put('aa', 'bb');
            return db;
        });
    }
    function withBufferStore() {
        return __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put(Buffer.from('aa'), Buffer.from('bb'));
            return db;
        });
    }
    function withDeepStore() {
        return __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put('aa', {
                'cc': 'dd',
                'ee': 'ff'
            });
            return db;
        });
    }
    describe(desc, () => {
        it('can put', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put('aa', 'bb');
        }));
        it('can put deep', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put('aa', {
                'cc': 'dd',
                'ee': 'ff'
            });
        }));
        it('can put key buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put(Buffer.from('aa'), 'bb');
        }));
        it('can put val buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put('aa', Buffer.from('bb'));
        }));
        it('can put key val buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield db.put(Buffer.from('aa'), Buffer.from('bb'));
        }));
        it('can get', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withStore();
            assert.deepEqual(yield db.get('aa', { asBuffer: false }), 'bb');
        }));
        it('can get deep', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withDeepStore();
            assert.deepEqual(yield db.get('aa', { asBuffer: false }), {
                'cc': 'dd',
                'ee': 'ff'
            });
        }));
        it('can get key buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withStore();
            assert.deepEqual(yield db.get(Buffer.from('aa'), { asBuffer: false }), 'bb');
        }));
        it('can get val buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withStore();
            assert.deepEqual(yield db.get('aa', { asBuffer: true }), Buffer.from('bb'));
        }));
        it('can get key val buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withStore();
            assert.deepEqual(yield db.get(Buffer.from('aa'), { asBuffer: true }), Buffer.from('bb'));
        }));
        it('can get from buffer store', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withBufferStore();
            assert.deepEqual(yield db.get('aa'), Buffer.from('bb'));
        }));
        it('can get key buffer from buffer store', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withBufferStore();
            assert.deepEqual(yield db.get(Buffer.from('aa')), Buffer.from('bb'));
        }));
        it('can get val buffer from buffer store', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withBufferStore();
            assert.deepEqual(yield db.get('aa', { asBuffer: true }), Buffer.from('bb'));
        }));
        it('can get key val buffer from buffer store', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withBufferStore();
            assert.deepEqual(yield db.get(Buffer.from('aa'), { asBuffer: true }), Buffer.from('bb'));
        }));
        it("can't get non-existant", () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield reverse_promise_1.reversePromise(db.get('aa', { asBuffer: false }));
        }));
        it('can del', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withStore();
            yield db.del('aa');
        }));
        it('can del deep', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withDeepStore();
            yield db.del('aa');
        }));
        it('can del buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withStore();
            yield db.del(Buffer.from('aa'));
        }));
        it('can del from buffer store', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withBufferStore();
            yield db.del('aa');
        }));
        it('can del buffer', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withBufferStore();
            yield db.del(Buffer.from('aa'));
        }));
        it("can't del non-existant", () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield reverse_promise_1.reversePromise(db.del('aa'));
        }));
        it('can batch/iterator', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            let validation = {
                'cc': 'dd',
                'ee': 'ff',
            };
            yield db.batch()
                .put('aa', 'bb')
                .put('cc', 'dd')
                .del('aa')
                .put('ee', 'ff')
                .write();
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
            }));
            for (const d of data) {
                assert.deepEqual(validation[d.key.toString()], d.value.toString());
                delete validation[d.key.toString()];
            }
            assert.deepEqual(validation, {});
        }));
        it('can iterate empty', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
            }));
            for (const d of data) {
                assert.fail("Got something, expected nothing");
            }
        }));
        it('can iterate while mutating', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            yield Promise.all([
                db.put('aa', 'bb'),
                new Promise((resolve, reject) => {
                    db.createReadStream({
                        keysAsBuffer: false,
                        valuesAsBuffer: false,
                    }).on('data', (data) => {
                    }).on('end', (data) => {
                        resolve();
                    });
                })
            ]);
        }));
        it('can batch/iterator with buffers', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            let validation = {
                [Buffer.from('cc')]: Buffer.from('dd'),
                [Buffer.from('ee')]: Buffer.from('ff'),
            };
            yield db.batch()
                .put(Buffer.from('aa'), Buffer.from('bb'))
                .put(Buffer.from('cc'), Buffer.from('dd'))
                .del(Buffer.from('aa'))
                .put(Buffer.from('ee'), Buffer.from('ff'))
                .write();
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: true,
                valuesAsBuffer: true,
            }));
            for (const d of data) {
                assert.deepEqual(validation[d.key], Buffer.from(d.value));
                delete validation[d.key];
            }
            assert.deepEqual(validation, {});
        }));
        it('can batch/iterator range', () => __awaiter(this, void 0, void 0, function* () {
            const db = yield withEmptyStore();
            let validation = {
                'cc': 'dd',
                'ee': 'ff',
            };
            yield db.batch()
                .put('aa', 'bb')
                .put('cc', 'dd')
                .put('ee', 'ff')
                .put('ff', 'qq')
                .write();
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
                gte: 'cc',
                lte: 'ee',
            }));
            for (const d of data) {
                assert.deepEqual(validation[String(d.key)], String(d.value));
                delete validation[d.key];
            }
            assert.deepEqual(validation, {});
        }));
    });
}
exports.CustomLevelDOWNTestSuite = CustomLevelDOWNTestSuite;
//# sourceMappingURL=custom-leveldown-test.js.map