"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = __importStar(require("assert"));
const levelup_1 = __importDefault(require("levelup"));
const subleveldown_1 = __importDefault(require("subleveldown"));
const index_1 = __importDefault(require("../index"));
const reverse_promise_1 = require("../util/reverse-promise");
const stream_promise_1 = require("../util/stream-promise");
// TODO: More stringent tests
function LevelMountTestSuite(desc, LevelDOWNFactory1, LevelDOWNFactory2) {
    function withEmptyStores() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                level_1: LevelDOWNFactory1(),
                level_2: LevelDOWNFactory2(),
            };
        });
    }
    describe(desc, () => {
        let level_1, level_2, db, db_no_mount, db_mount;
        before(() => __awaiter(this, void 0, void 0, function* () {
            const stores = yield withEmptyStores();
            level_1 = levelup_1.default(stores.level_1);
            level_2 = levelup_1.default(stores.level_2);
            db = levelup_1.default(index_1.default(stores.level_1, { '!mount!': stores.level_2 }));
            db_no_mount = subleveldown_1.default(db, '!no_mount!', { separator: '!' });
            db_mount = subleveldown_1.default(db, '!mount!', { separator: '!' });
            yield db.put('aa', 'bb');
            yield db_no_mount.put('cc', 'dd');
            yield db_mount.put('ee', 'ff');
        }));
        it('can access level_1', () => __awaiter(this, void 0, void 0, function* () {
            assert.equal(yield level_1.get('aa'), 'bb');
        }));
        it('can access no_mount', () => __awaiter(this, void 0, void 0, function* () {
            assert.equal(yield db_no_mount.get('cc'), 'dd');
        }));
        it('can access mount', () => __awaiter(this, void 0, void 0, function* () {
            assert.equal(yield db_mount.get('ee'), 'ff');
        }));
        it('can access mount through db', () => __awaiter(this, void 0, void 0, function* () {
            assert.equal(yield db.get('!mount!ee'), 'ff');
        }));
        it('can access no_mount through level_1', () => __awaiter(this, void 0, void 0, function* () {
            assert.equal(yield level_1.get('!no_mount!cc'), 'dd');
        }));
        it('can not access mount through level_1', () => __awaiter(this, void 0, void 0, function* () {
            yield reverse_promise_1.reversePromise(level_1.get('!mount!ee'));
        }));
        it('can access mount through level_2', () => __awaiter(this, void 0, void 0, function* () {
            assert.equal(yield level_2.get('ee'), 'ff');
        }));
        it('can iterate through all stores', () => __awaiter(this, void 0, void 0, function* () {
            let validation = {
                'aa': 'bb',
                '!no_mount!cc': 'dd',
                '!mount!ee': 'ff',
            };
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
            }));
            for (const d of data) {
                assert.deepEqual(validation[String(d.key)], String(d.value));
                delete validation[String(d.key)];
            }
            assert.deepEqual(validation, {});
        }));
        it('can iterate through multiple stores', () => __awaiter(this, void 0, void 0, function* () {
            let validation = {
                '!no_mount!cc': 'dd',
                '!mount!ee': 'ff',
            };
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
                gt: '!',
                lt: '!\xff',
            }));
            for (const d of data) {
                assert.deepEqual(validation[String(d.key)], String(d.value));
                delete validation[String(d.key)];
            }
            assert.deepEqual(validation, {});
        }));
        it('can iterate through specific store', () => __awaiter(this, void 0, void 0, function* () {
            let validation = {
                '!mount!ee': 'ff',
            };
            const data = yield stream_promise_1.streamPromise(db.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
                gt: '!mount!',
                lt: '!mount!\xff',
            }));
            for (const d of data) {
                assert.deepEqual(validation[String(d.key)], String(d.value));
                delete validation[String(d.key)];
            }
            assert.deepEqual(validation, {});
        }));
        it('can iterate through specific store with sublevel', () => __awaiter(this, void 0, void 0, function* () {
            let validation = {
                'ee': 'ff',
            };
            const data = yield stream_promise_1.streamPromise(db_mount.createReadStream({
                keysAsBuffer: false,
                valuesAsBuffer: false,
            }));
            for (const d of data) {
                assert.deepEqual(validation[String(d.key)], String(d.value));
                delete validation[String(d.key)];
            }
            assert.deepEqual(validation, {});
        }));
    });
}
exports.LevelMountTestSuite = LevelMountTestSuite;
//# sourceMappingURL=level-mount-test.js.map