"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tape_1 = __importDefault(require("tape"));
const assert_1 = __importDefault(require("assert"));
const stream_promise_1 = require("./stream-promise");
/**
 * Convert tape tests into mocha assertions!
 *
 * @example
 * tape_it('my mocha tape test assertion', function (test) {
 *   test('my tape test', function (t) {
 *     t.equal(1, 1)
 *   })
 * })
 *
 * @param desc Description of the test (like normal `it`)
 * @param func Test function accepting test harness as parameter
 * @returns Same the resulting `it` mocha construct
 */
function tape_it(desc, func) {
    return it(desc, () => __awaiter(this, void 0, void 0, function* () {
        const htest = tape_1.default.createHarness();
        const stream = htest.createStream();
        const data_promise = stream_promise_1.streamPromise(stream);
        const htest_promise = func(htest);
        let name;
        for (const row of yield data_promise) {
            if (row.indexOf('#') === 0)
                name = row.slice(2);
            if (row.indexOf('ok') === 0)
                assert_1.default.equal(true, true, name);
            else if (row.indexOf('not ok') === 0)
                assert_1.default.equal(true, false, name);
        }
        yield htest_promise;
    }));
}
exports.default = tape_it;
//# sourceMappingURL=tape_it.js.map